@@ -1,5 +1,5 @@
 // dwarf location-list-to-c translator
-// Copyright (C) 2005-2013 Red Hat Inc.
+// Copyright (C) 2005-2014 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -22,6 +22,9 @@
 
 #include "config.h"
 
+#if ! _ELFUTILS_PREREQ(0, 153)
+#define DW_OP_GNU_entry_value 0xf3
+#endif
 
 #define N_(x) x
 
@@ -858,7 +861,11 @@ translate (struct location_context *ctx, int indent,
 	  break;
 
 	case DW_OP_push_object_address:
-	  DIE ("XXX DW_OP_push_object_address");
+	  DIE ("unhandled DW_OP_push_object_address");
+	  break;
+
+	case DW_OP_GNU_entry_value:
+	  DIE ("unhandled DW_OP_GNU_entry_value");
 	  break;
 
 	default:
@@ -1364,7 +1371,11 @@ location_relative (struct location_context *ctx, int indent,
 	  break;
 
 	case DW_OP_push_object_address:
-	  DIE ("XXX DW_OP_push_object_address");
+	  DIE ("unhandled DW_OP_push_object_address");
+	  break;
+
+	case DW_OP_GNU_entry_value:
+	  DIE ("unhandled DW_OP_GNU_entry_value");
 	  break;
 
 	default:
@@ -2232,7 +2243,8 @@ pointer_stride (Dwarf_Die *typedie, struct location *origin)
   int typetag = dwarf_tag (&die_mem);
   while (typetag == DW_TAG_typedef ||
 	 typetag == DW_TAG_const_type ||
-	 typetag == DW_TAG_volatile_type)
+	 typetag == DW_TAG_volatile_type ||
+	 typetag == DW_TAG_restrict_type)
     {
       if (dwarf_attr_integrate (&die_mem, DW_AT_type, &attr_mem) == NULL
 	  || dwarf_formref_die (&attr_mem, &die_mem) == NULL)
