@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2010 Red Hat Inc.
+// Copyright (C) 2005-2014 Red Hat Inc.
 // Copyright (C) 2007 Bull S.A.S
 //
 // This file is part of systemtap, and is free software.  You can
@@ -17,6 +17,7 @@
 #include <stdexcept>
 
 struct stapfile;
+struct probe;
 
 struct source_loc
 {
@@ -48,6 +49,13 @@ struct token
   std::string msg; // for tok_junk
   void make_junk (std::string msg);
   const token* chain; // macro invocation that produced this token
+  friend class parser;
+  friend class lexer;
+private:
+  token(): chain(0) {}
+  token(const token& other):
+    location(other.location), type(other.type), content(other.content),
+    msg(other.msg), chain(other.chain) {}
 };
 
 
@@ -73,11 +81,21 @@ struct macrodecl {
 };
 
 
-stapfile* parse (systemtap_session& s, std::istream& i, bool privileged);
-stapfile* parse (systemtap_session& s, const std::string& n, bool privileged);
+enum parse_flag
+  {
+    pf_guru = 1,
+    pf_no_compatible = 2,
+    pf_squash_errors = 4,
+  };
+
+
+stapfile* parse (systemtap_session& s,const std::string& n, std::istream& i, unsigned flags);
+stapfile* parse (systemtap_session& s, const std::string& n, unsigned flags);
 
 stapfile* parse_library_macros (systemtap_session& s, const std::string& n);
 
+probe* parse_synthetic_probe (systemtap_session &s, std::istream& i, const token* tok);
+
 #endif // PARSE_H
 
 /* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
