@@ -1,6 +1,7 @@
 /* -*- linux-c -*- 
  * time-estimation with minimal dependency on xtime
  * Copyright (C) 2006 Intel Corporation.
+ * Copyright (C) 2010-2014 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -21,6 +22,24 @@
 
 #include <linux/cpufreq.h>
 
+/* The interval at which the __stp_time_timer_callback routine runs,
+   which resynchronizes our per-cpu base_ns/base_cycles values.  A
+   lower rate (higher interval) is sufficient if we get cpufreq-change
+   notifications. */
+#ifndef STP_TIME_SYNC_INTERVAL_NOCPUFREQ
+#define STP_TIME_SYNC_INTERVAL_NOCPUFREQ  ((HZ+9)/10) /* ten times per second */
+#endif
+#ifndef STP_TIME_SYNC_INTERVAL_CPUFREQ
+#define STP_TIME_SYNC_INTERVAL_CPUFREQ    (HZ*10)     /* once per ten seconds */
+#endif
+static int __stp_cpufreq_notifier_registered = 0;
+
+#ifndef STP_TIME_SYNC_INTERVAL
+#define STP_TIME_SYNC_INTERVAL (__stp_cpufreq_notifier_registered ? \
+                                STP_TIME_SYNC_INTERVAL_CPUFREQ : \
+                                STP_TIME_SYNC_INTERVAL_NOCPUFREQ)
+#endif
+
 #ifndef NSEC_PER_MSEC
 #define NSEC_PER_MSEC	1000000L
 #endif
@@ -72,7 +91,7 @@ __stp_get_freq(void)
     // If we can get the actual frequency of HW counter, we use it.
 #if defined (__ia64__)
     return local_cpu_data->itc_freq / 1000;
-#elif defined (__s390__) || defined (__s390x__) || defined (__arm__)
+#elif defined (__s390__) || defined (__s390x__) || defined (__arm__) || defined (__aarch64__)
     // We don't need to find the cpu freq on s390 as the 
     // TOD clock is always a fix freq. (see: POO pg 4-36.)
     return 0;
@@ -80,7 +99,7 @@ __stp_get_freq(void)
     return tsc_khz;
 #elif (defined (__i386__) || defined (__x86_64__)) && defined(STAPCONF_CPU_KHZ)
     return cpu_khz;
-#else /* __i386__ || __x86_64__ || __ia64__ || __s390__ || __s390x__ || __arm__*/
+#else /* __i386__ || __x86_64__ */
     // If we don't know the actual frequency, we estimate it.
     cycles_t beg, mid, end;
     beg = get_cycles(); barrier();
@@ -105,9 +124,11 @@ __stp_ktime_get_real_ts(struct timespec *ts)
 #endif
 }
 
-/* The timer callback is in a softIRQ -- interrupts enabled. */
-static void
-__stp_time_timer_callback(unsigned long val)
+
+/* Update this cpu's base_ns/base_cycles values.  May be called from
+   initialization or various other callback mechanisms. */
+static stp_time_t*
+__stp_time_local_update(void)
 {
     unsigned long flags;
     stp_time_t *time;
@@ -119,22 +140,48 @@ __stp_time_timer_callback(unsigned long val)
 
     __stp_ktime_get_real_ts(&ts);
     cycles = get_cycles();
-
     ns = (NSEC_PER_SEC * (int64_t)ts.tv_sec) + ts.tv_nsec;
-
     time = per_cpu_ptr(stp_time, smp_processor_id());
+
     write_seqlock(&time->lock);
     time->base_ns = ns;
     time->base_cycles = cycles;
     write_sequnlock(&time->lock);
 
     local_irq_restore(flags);
+
+    return time;
+}
+
+
+
+/* The cross-smp call. */
+static void
+__stp_time_smp_callback(void *val)
+{
+    (void) val;
+    (void) __stp_time_local_update();
+}
+
+
+/* The timer callback is in a softIRQ -- interrupts enabled. */
+static void
+__stp_time_timer_callback(unsigned long val)
+{
+    stp_time_t *time =__stp_time_local_update();
+    (void) val;
+
     /* PR6481: reenable IRQs before resetting the timer.
        XXX: The worst that can probably happen is that we get
 	    two consecutive timer resets.  */
 
     if (likely(atomic_read(session_state()) != STAP_SESSION_STOPPED))
-        mod_timer(&time->timer, jiffies + 1);
+        mod_timer(&time->timer, jiffies + STP_TIME_SYNC_INTERVAL);
+
+#ifdef DEBUG_TIME
+    _stp_warn("cpu%d %p khz=%d base=%lld cycles=%lld\n", smp_processor_id(), (void*)time, time->freq,
+              (long long)time->base_ns, (long long)time->base_cycles);
+#endif
 }
 
 /* This is called as an IPI, with interrupts disabled. */
@@ -145,13 +192,11 @@ __stp_init_time(void *info)
     stp_time_t *time = per_cpu_ptr(stp_time, smp_processor_id());
 
     seqlock_init(&time->lock);
-    __stp_ktime_get_real_ts(&ts);
-    time->base_cycles = get_cycles();
-    time->base_ns = (NSEC_PER_SEC * (int64_t)ts.tv_sec) + ts.tv_nsec;
     time->freq = __stp_get_freq();
+    __stp_time_local_update();
 
     init_timer(&time->timer);
-    time->timer.expires = jiffies + 1;
+    time->timer.expires = jiffies + STP_TIME_SYNC_INTERVAL;
     time->timer.function = __stp_time_timer_callback;
 
 #ifndef STAPCONF_ADD_TIMER_ON
@@ -169,16 +214,43 @@ __stp_time_cpufreq_callback(struct notifier_block *self,
     struct cpufreq_freqs *freqs;
     int freq_khz;
     stp_time_t *time;
+    struct timespec ts;
+    int64_t ns;
+    cycles_t cycles;
+    int reset_timer_p = 0;
 
     switch (state) {
         case CPUFREQ_POSTCHANGE:
+#ifdef CPUFREQ_RESUMECHANGE
         case CPUFREQ_RESUMECHANGE:
+#endif
             freqs = (struct cpufreq_freqs *)vfreqs;
             freq_khz = freqs->new;
             time = per_cpu_ptr(stp_time, freqs->cpu);
             write_seqlock_irqsave(&time->lock, flags);
-            time->freq = freq_khz;
+            if (time->freq != freq_khz) {
+                    time->freq = freq_khz;
+                    // NB: freqs->cpu may not equal smp_processor_id(),
+                    // so we can't update the subject processor's
+                    // base_ns/base_cycles values just now.
+                    reset_timer_p = 1;
+            }
             write_sequnlock_irqrestore(&time->lock, flags);
+            if (reset_timer_p) {
+#ifdef DEBUG_TIME
+                    _stp_warn ("cpu%d %p freq->%d\n", freqs->cpu, (void*)time, freqs->new);
+#endif
+#if defined(STAPCONF_SMPCALL_5ARGS) || defined(STAPCONF_SMPCALL_4ARGS)
+                    (void) smp_call_function_single (freqs->cpu, &__stp_time_smp_callback, 0,
+#ifdef STAPCONF_SMPCALL_5ARGS
+                                                     1, /* nonatomic */
+#endif
+                                                     0); /* not wait */
+#else
+                    /* RHEL4ish: cannot direct to a single cpu ... so broadcast to them all */
+                    (void) smp_call_function (&__stp_time_smp_callback, NULL, 0, 0);
+#endif
+            }
             break;
     }
 
@@ -215,7 +287,7 @@ _stp_kill_time(void)
             del_timer_sync(&time->timer);
         }
 #ifdef CONFIG_CPU_FREQ
-        if (!__stp_constant_freq()) {
+        if (!__stp_constant_freq() && __stp_cpufreq_notifier_registered) {
             cpufreq_unregister_notifier(&__stp_time_notifier,
                                         CPUFREQ_TRANSITION_NOTIFIER);
         }
@@ -253,13 +325,12 @@ _stp_init_time(void)
 
 #ifdef CONFIG_CPU_FREQ
     if (!ret && !__stp_constant_freq()) {
-        ret = cpufreq_register_notifier(&__stp_time_notifier,
-                CPUFREQ_TRANSITION_NOTIFIER);
-
-        if (!ret) {
+	if (!cpufreq_register_notifier(&__stp_time_notifier,
+				       CPUFREQ_TRANSITION_NOTIFIER)) {
+	    __stp_cpufreq_notifier_registered = 1;
             for_each_online_cpu(cpu) {
                 unsigned long flags;
-                int freq_khz = cpufreq_get(cpu);
+                int freq_khz = cpufreq_get(cpu); // may block
                 if (freq_khz > 0) {
                     stp_time_t *time = per_cpu_ptr(stp_time, cpu);
                     write_seqlock_irqsave(&time->lock, flags);
