@@ -86,6 +86,7 @@ itrace_derived_probe::join_group (systemtap_session& s)
     s.itrace_derived_probes = new itrace_derived_probe_group ();
 
   s.itrace_derived_probes->enroll (this);
+  this->group = s.itrace_derived_probes;
 
   enable_task_finder(s);
 }
@@ -117,6 +118,12 @@ struct itrace_builder: public derived_probe_builder
         sess.unwindsym_modules.insert (path);
         path_tgt = path_remove_sysroot(sess, path);
       }
+    else // (has_pid)
+      {
+	string pid_err_msg;
+	if (!is_valid_pid(pid, pid_err_msg))
+	  throw SEMANTIC_ERROR(pid_err_msg);
+      }
 
     finished_results.push_back(new itrace_derived_probe(sess, base, location,
 							has_path, path_tgt, pid,
@@ -197,7 +204,7 @@ itrace_derived_probe_group::emit_module_decls (systemtap_session& s)
 
   // call probe function
   s.op->newline() << "(*p->probe->ph) (c);";
-  common_probe_entryfn_epilogue (s, true);
+  common_probe_entryfn_epilogue (s, true, otf_safe_context(s));
 
   s.op->newline() << "return;";
   s.op->newline(-1) << "}";
