@@ -16,6 +16,11 @@ int main()
   //staptest// pipe (\[0, 0\]) = 0
 
 #ifdef O_CLOEXEC
+  /* Test calling pipe2() with 0, to make sure we don't confuse it
+   * with pipe(). */
+  pipe2 (pipefd, 0);
+  //staptest// pipe2 (\[NNNN, NNNN\], 0) = 0
+
   pipe2 (pipefd, O_CLOEXEC);
   //staptest// pipe2 (\[NNNN, NNNN\], O_CLOEXEC) = 0
 
