@@ -19,8 +19,10 @@ inline int ibar (void) {
   return libloopfunc ();
 }
 
-/* We need a threaded app. */
-inline int tbar (void) {
+/* We need a threaded app.  It should not be inline,
+   so a .callee("tbar") test will require the GNU_call_site* DWARF goo
+   to match, not just the presence of a mere DW_AT_inline=3 case. */
+int tbar (void) {
   void *x;
   int j = 0;
   STAP_PROBE(_test_, main_enter);
@@ -34,13 +36,20 @@ inline int tbar (void) {
 
 main (int argc, char *argv[]) {
   int j = 0;
+  /* Don't loop if an argument was passed */
+  /* We split into two cases rather than check in a single loop because
+   * GCC can output screwy debuginfo otherwise (RHBZ1092144) */
+  if (argc > 1) {
+    j += ibar ();
+    j += tbar ();
+    return 0;
+  }
   for (;;) {
     j += ibar ();
     j += tbar ();
-    /* Don't loop if an argument was passed */
-    if (argc > 1)
-      return 0;
+#if !defined NOSLEEP
     usleep (250000); /* 1/4 second pause.  */
+#endif
   }
   return j;
 }
